#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Tests for the functionality of the Notification REST API.
The logic can be found in `notification/views/notify.py`.
"""


# pylint: disable=C0415, E0401, R0201

# TODO: re-enable after tests are fixed
# pylint: disable=W0511

# TODO: re-enable after pydoc has been added
# pylint: disable=C0116
import copy
import http
from unittest.mock import patch

import pytest
from pyramid.testing import DummyRequest


@pytest.mark.usefixtures("dummy_send", "mock_notification_service")
class TestNotifyViews:
    """Unit tests for notification endpoints"""

    def test_view_notifications(self, dummy_send: DummyRequest):
        from notification.views.notify import NotificationListRestService

        response = NotificationListRestService(dummy_send).list_notification_templates()
        assert len(response) == 1
        # TODO: create a mock list of notification templates, and make sure that they're all found

    def test_send_notification(self, dummy_send: DummyRequest):
        from notification.views.notify import NotificationRestService

        dummy_send.json_body = {
            "destination_email": "chausman@nrao.edu",
            "subject": "first thing",
            "message": "second thing",
        }

        dummy_send.matchdict["template_name"] = "email"
        with patch(
            "workspaces.notification.services.notification_service.NotificationService.send_email"
        ) as send_email:
            send_email.return_value = {}
            response = NotificationRestService(dummy_send).send_notification()
        assert response["errors"] == {}
        assert response["message"] == "Email sent"

    def test_send_notification_with_list(self, dummy_send: DummyRequest):
        from notification.views.notify import NotificationRestService

        dummy_send.json_body = {
            "destination_email": ["chausman@nrao.edu", "dreamerofthestars@gmail.com"],
            "subject": "first thing",
            "message": "second thing",
        }

        dummy_send.matchdict["template_name"] = "email"
        with patch(
            "workspaces.notification.services.notification_service.NotificationService.send_email"
        ) as send_email:
            send_email.return_value = {}
            response = NotificationRestService(dummy_send).send_notification()
        assert response["errors"] == {}
        assert response["message"] == "Email sent"

    def test_create_notification(self, dummy_send: DummyRequest):
        """
        Can we create a notification template?

        :param dummy_send:
        :return:
        """
        from notification.views.notify import NotificationRestService

        dummy_create = copy.copy(dummy_send)

        template_name = "oh_no_not_another_template"
        dummy_create.json_body = {
            "template_name": template_name,
            "description": "a test template",
            "content": "Subject: {{foo}}\n\n{{bar}}",
        }

        created = NotificationRestService(dummy_create).create_notification_template()
        assert created.content == "Subject: {{foo}}\n\n{{bar}}"

        dummy_create.matchdict["template_name"] = template_name
        response = NotificationRestService(dummy_create).delete_notification_template()
        assert response.status_code == http.HTTPStatus.OK

        # if we can update it, we know it's been created
        dummy_update = copy.copy(dummy_create)
        dummy_update.json_body["template_name"] = template_name
        dummy_update.json_body["template_text"] = "my little pony"
        response = NotificationRestService(dummy_update).update_notification_template()
        assert response.status_code == http.HTTPStatus.OK

        NotificationRestService(dummy_update).delete_notification_template()

    def test_delete_notification(self, dummy_send: DummyRequest):
        from notification.views.notify import NotificationRestService

        template_name = "delete_me"
        dummy_send.matchdict["template_name"] = template_name

        response = NotificationRestService(dummy_send).delete_notification_template()
        assert response.status_code == http.HTTPStatus.OK

        # TODO: confirm the thing really was deleted

    def test_update_notification_template(self, dummy_send: DummyRequest):
        from notification.views.notify import NotificationRestService

        template_name = "email"
        template_text = "Subject: {{foo}}\n\n{{bar}}\nTo:\n\n{{baz}}"
        dummy_send.json_body["template_name"] = template_name
        dummy_send.json_body["template_text"] = template_text

        response = NotificationRestService(dummy_send).update_notification_template()
        assert response.status_code == http.HTTPStatus.OK

        # TODO: did template text -really- get updated?
